﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;
using System.Windows.Forms;

namespace ProgrammingLanguagesConverter
{
    //перечисление типов лексем
    enum LexemType
    {
        Divider, //разделители
        Keyword, //ключевые слова
        Variable, //переменные
        VarType //объявление типа
    }

    //перечисление подтипов лексем
    enum LexemSubType
    {
        any, //любой
		 
        variable, //переменная
		constant, //константа

        //***************указатели на разделители******************************//
        endOfLine, //указатель конца строки
        spaceSymbol, //указатель разделения слов
        stampleOpenSymbol, //указатель открытия скобок
        stampleCloseSymbol, //указатель закрытия скобок
        squareStampleOpen,
        squareStampleClose,
        colonSymbol, //двоеточие
		commaSymbol, //запятая
        arrayRange, //символ, указывающий границы массива
		lineComment, //строчный комментарий
		blockCommentOpen,
		blockCommentClose, //блочный комментарий
        stringOpen,
        charOpen,
		programEnd, //указатель конца программы
		mathPlus, //плюс
		mathMinus, //минус
		mathInc, //инкремент
		mathDec, //декремент
		mathMultiply, //умножение
		mathDivide, //деление
		mathEqual, //равно
		mathUnequal, //неравно
		mathMore, //больше
		mathLess, //меньше
		carriageReturn, //возврат каретки

        //***************указатели на блоки************************************//
        programStart, //указатель начала программы
        varBlock, //указатель блока переменных
        constDef,
        blockStart, //указатель начала блока
        blockEnd, //указатель конца блока
        procedure,
        function,

        //***************указатели на типы переменных**************************//
		typeVoid,
        type8sInt, //указатель на целую 8-битную переменную со знаком
        type8usInt, //указатель на целую 8-битную переменную без знака
        type16sInt, //указатель на целую 16-битную переменную со знаком
        type16usInt, //указатель на целую 16-битную переменную без знака
        type32sInt, //указатель на целую 32-битную переменную со знаком
        type32usInt, //указатель на целую 32-битную переменную без знака
        type64sInt, //указатель на целую 64-битную переменную со знаком
        type64usInt, //указатель на целую 64-битную переменную без знака
        typeChar, //указатель на символьную переменную
        type32Float, //указатель на 32-битную с плавующей точкой
        type64Float, //указатель на 64-битную с плавующей точкой
		typeString,
        
        //***************указатели направления алгоритма***********************//
        algFor, 
        algTo,
        algDo,
        algNext,
        algIf,
        algThen,
        algElse,
        algEnd,
        algWhile,
        algRepeat,
        algUntil,
        algLoop,
        algWend,
        algDownto,
        algSwitch,
        algCase,
        algOf,
        arrayType,

        //***************указатели на логические операции*******************//
        logicAnd, //и
        logicOr, //или
        logicNot, //не
		logicAssign, //присвоить
        logicAs,
        preProc,

        read,
        readln,
        write,
        writeln,

		returnWord //возвращение значения функции
    }

    //структура, описывающая лексему
    struct Lexem
    {
        public string Content; //сама лексема
        public LexemType Type; //ее тип
        public LexemSubType SubType; //подтип лексемы
        public string Ambiguousity; //однозначность

        public Lexem(string content, LexemType type, LexemSubType subType)
        {
            this.Content = content; 
            this.Type = type;
            this.SubType = subType;
			this.Ambiguousity = null;
        }

		public Lexem(char content, LexemType type, LexemSubType subType)
        {
            this.Content = "";
            this.Content += content;
            this.Type = type;
            this.SubType = subType;
			this.Ambiguousity = null;
        }

        public void SetType(LexemType lt)
        {
            this.Type = lt;
        }

        public void SetSubType(LexemSubType lst)
        {
            this.SubType = lst;
        }
    }

	//класс конвертируемого языка
    class ConvertableLanguage
    {
	    public string Name { get; private set; } //название

	    public List<Lexem> MainLexems { get; private set; } //основные лексемы
		public List<Lexem> ForLoopLexems { get; private set; } //последовательность объявления цикла со счетчиком

		public bool BVarDefIsType { get; private set; }
        public bool BGapInArr { get; private set; }
		public bool BStrictlyStamples { get; private set; }
		public bool BNeedCloser { get; private set; }
        public bool BStrictlyBlockInRepeat { get; private set; }
        public bool BInversedRepeat { get; private set; }

	    public ConvertableLanguage()
		{
			MainLexems = new List<Lexem>();
			ForLoopLexems = new List<Lexem>();
		}

		//функция загрузки лексем из файла
		public void LoadLexemsFromFile(string path)
		{
			var xmlTr = new XmlTextReader(new FileStream(path, FileMode.Open)); 

			bool bReadingLexems = false; //флаг чтения лексем
			bool bReadingForLoop = false; //флаг чтения цикла со счетчиком
			bool bReadingName = false; //флаг чтения имени
			var bufLexem = new Lexem(); //лексема-буфер
			var readingLexemType = new LexemType(); //буфер чтения типа лексемы

            MainLexems.Clear();
            ForLoopLexems.Clear();

			//пока не кончился файл
			while (xmlTr.Read())
			{
				//проверяем тип ноды
				switch (xmlTr.NodeType)
				{
					//если элемент
					case XmlNodeType.Element:
						//проверяем имя
						switch (xmlTr.Name)
						{
							//ставим соответствующий флаг
							case "Lexems":
								bReadingLexems = true;
								break;
							case "VariableDefinition":
                                if (xmlTr.GetAttribute("type") == "type")
                                    BVarDefIsType = true;
                                else
                                    BVarDefIsType = false;
								break;
                            case "ArrayDefinition":
                                if (xmlTr.GetAttribute("gap") == "true")
                                    BGapInArr = true;
                                else
                                    BGapInArr = false;
                                break;
							case "OperatorClosers":
                                if (xmlTr.GetAttribute("needStamples") == "strictly")
                                    BStrictlyStamples = true;
                                else
                                    BStrictlyStamples = false;

                                if (xmlTr.GetAttribute("needCloser") == "true")
                                    BNeedCloser = true;
                                else
                                    BNeedCloser = false;
								break;
                            case "RepeatLoopConstruction":
                                if (xmlTr.GetAttribute("needBlock") == "strictly")
                                    BStrictlyBlockInRepeat = true;
                                else
                                    BStrictlyBlockInRepeat = false;

                                if (xmlTr.GetAttribute("reversed") == "true")
                                    BInversedRepeat = true;
                                else
                                    BInversedRepeat = false;
                                break;
							case "ForLoopConstruction":
								bReadingForLoop = true;
								break;
							case "Dividers":
								readingLexemType = LexemType.Divider;
								break;
							case "Keywords":
								readingLexemType = LexemType.Keyword;
								break;
							case "VarTypes":
								readingLexemType = LexemType.VarType;
								break;
							case "Name":
								bReadingName = true;
								break;
							default:
								//если уже идет чтение, выполняем соответсвующие действия
								if (bReadingLexems)
								{
									bufLexem = new Lexem {SubType = GetLexemSubTypeFromXml(xmlTr.Name), Type = readingLexemType};
									if (xmlTr.GetAttribute("ambiguous") != null)
									{
										bufLexem.Ambiguousity = xmlTr.GetAttribute("ambiguous");
									}
								}
								else if (bReadingForLoop)
								{
									bufLexem = new Lexem(null, readingLexemType, GetLexemSubTypeFromXml(xmlTr.Name));
									ForLoopLexems.Add(bufLexem);
								}
								break;
						}
						break;
					case XmlNodeType.Text:
						if (bReadingLexems)
						{
							bufLexem.Content = xmlTr.Value;
							MainLexems.Add(bufLexem);
						}
						else if (bReadingName)
						{
							Name = xmlTr.Value;
							bReadingName = false;
						}

                        if (xmlTr.Value == "\n")
                        {
                            MainLexems.Add(new Lexem("\n", LexemType.Divider, LexemSubType.endOfLine));
                        }
						break;
					case XmlNodeType.EndElement:
						if (xmlTr.Name == "Lexems")
						{
							bReadingLexems = false;
						}
						else if (xmlTr.Name == "ForLoopConstruction")
						{
							bReadingForLoop = false;
						}
						break;
				}
			}

			xmlTr.Close();
		}

		private LexemSubType GetLexemSubTypeFromXml(string xmlName)
		{
			LexemSubType subtype = LexemSubType.any;            

			switch (xmlName)
			{
				case "Variable":
					subtype = LexemSubType.variable;
					break;
				case "Constant":
					subtype = LexemSubType.constant;
					break;

				//Dividers
				case "EndOfLine":
					subtype = LexemSubType.endOfLine;
					break;
				case "SpaceSymbol":
					subtype = LexemSubType.spaceSymbol;
					break;
				case "StampleOpenSymbol":
					subtype = LexemSubType.stampleOpenSymbol;
					break;
				case "StampleCloseSymbol":
					subtype = LexemSubType.stampleCloseSymbol;
					break;
                case "SquareStampleOpen":
                    subtype = LexemSubType.squareStampleOpen;
                    break;
                case "SquareStampleClose":
                    subtype = LexemSubType.squareStampleClose;
                    break;
				case "ColonSymbol":
					subtype = LexemSubType.colonSymbol;
					break;
				case "CommaSymbol":
					subtype = LexemSubType.commaSymbol;
					break;
                case "ArrayRange":
                    subtype = LexemSubType.arrayRange;
                    break;
				case "MathPlus":
					subtype = LexemSubType.mathPlus;
					break;
                case "MathMinus":
                    subtype = LexemSubType.mathMinus;
                    break;
				case "MathEqual":
					subtype = LexemSubType.mathEqual;
					break;
                case "MathUnequal":
                    subtype = LexemSubType.mathUnequal;
                    break;
				case "LogicAssign":
					subtype = LexemSubType.logicAssign;
					break;
                case "LogicAs":
                    subtype = LexemSubType.logicAs;
                    break;
				case "LineComment":
					subtype = LexemSubType.lineComment;
					break;
                case "BlockCommentOpen":
                    subtype = LexemSubType.blockCommentOpen;
                    break;
                case "BlockCommentClose":
                    subtype = LexemSubType.blockCommentClose;
                    break;
                case "StringOpen":
                    subtype = LexemSubType.stringOpen;
                    break;
                case "CharOpen":
                    subtype = LexemSubType.charOpen;
                    break;
				case "MathLess":
					subtype = LexemSubType.mathLess;
					break;
				case "MathMore":
					subtype = LexemSubType.mathMore;
					break;
				case "MathInc":
					subtype = LexemSubType.mathInc;
					break;
                case "PreProc":
                    subtype = LexemSubType.preProc;
                    break;

				//Keywords
				case "ProgramStart":
					subtype = LexemSubType.programStart;
					break;
				case "ProgramEnd":
					subtype = LexemSubType.programEnd;
					break;
				case "VarBlock":
					subtype = LexemSubType.varBlock;
					break;
                case "ConstDef":
                    subtype = LexemSubType.constDef;
                    break;
				case "BlockStart":
					subtype = LexemSubType.blockStart;
					break;
				case "BlockEnd":
					subtype = LexemSubType.blockEnd;
					break;
				case "AlgIf":
					subtype = LexemSubType.algIf;
					break;
				case "AlgThen":
					subtype = LexemSubType.algThen;
					break;
				case "AlgElse":
					subtype = LexemSubType.algElse;
					break;
                case "AlgEnd":
                    subtype = LexemSubType.algEnd;
                    break;
				case "AlgFor":
					subtype = LexemSubType.algFor;
					break;
				case "AlgTo":
					subtype = LexemSubType.algTo;
					break;
				case "AlgDo":
					subtype = LexemSubType.algDo;
					break;
                case "AlgNext":
                    subtype = LexemSubType.algNext;
                    break;
                case "AlgWhile":
                    subtype = LexemSubType.algWhile;
                    break;
                case "AlgRepeat":
                    subtype = LexemSubType.algRepeat;
                    break;
                case "AlgUntil":
                    subtype = LexemSubType.algUntil;
                    break;
                case "AlgLoop":
                    subtype = LexemSubType.algLoop;
                    break;
                case "AlgWend":
                    subtype = LexemSubType.algWend;
                    break;
                case "Procedure":
                    subtype = LexemSubType.procedure;
                    break;
                case "Function":
                    subtype = LexemSubType.function;
                    break;
                case "ReturnWord":
                    subtype = LexemSubType.returnWord;
                    break;
				case "LogicAnd":
					subtype = LexemSubType.logicAnd;
					break;
                case "ArrayType":
                    subtype = LexemSubType.arrayType;
                    break;
                case "AlgOf":
                    subtype = LexemSubType.algOf;
                    break;
                case "Read":
                    subtype = LexemSubType.read;
                    break;
                case "ReadLine":
                    subtype = LexemSubType.readln;
                    break;
                case "Write":
                    subtype = LexemSubType.write;
                    break;
                case "WriteLine":
                    subtype = LexemSubType.writeln;
                    break;

				//VarTypes
                case "TypeVoid":
                    subtype = LexemSubType.typeVoid;
                    break;
				case "Type32sInt":
					subtype = LexemSubType.type32sInt;
					break;
				case "TypeChar":
					subtype = LexemSubType.typeChar;
					break;
				case "Type32Float":
					subtype = LexemSubType.type32Float;
					break;
				case "TypeString":
					subtype = LexemSubType.typeString;
					break;
			}
			
			return subtype;
		}

		public void ShowLexemsInConsole()
		{
			string lexemInfo = null;

			lexemInfo += Name + "\n\n";
			lexemInfo += "******Main******\n";

			foreach (Lexem lex in MainLexems)
			{
				lexemInfo += lex.Content + " -content " + lex.SubType.ToString() + " -subType " + lex.Type + " -type " + lex.Ambiguousity + " -amb\n";
			}

			lexemInfo += "\n******ForLoop******\n";

			foreach (Lexem lex in ForLoopLexems)
			{
				lexemInfo += lex.Content + " -content " + lex.SubType.ToString() + " -subType " + lex.Type + " -type " + lex.Ambiguousity + " -amb\n";
			}

			MessageBox.Show(lexemInfo);
		}
    }
}
